#include <pthread.h>
#include <semaphore.h>

#ifndef MEMORYMANAGER_H
#define MEMORYMANAGER_H

struct Frame
{
  char * frameData;
};

typedef struct Frame Frame;

struct PhysicalRam
{
  int * used;
  Frame * frames;
};

typedef struct PhysicalRam PhysicalRam;

struct Page
{
  int pageSize;
  Frame * frame;
};

typedef struct Page Page;

struct Segment
{
  int threadOwner;
  Page * pageList;
  int numPages;
  struct Segment * next;
};

typedef struct Segment Segment;

struct VirtualHDD
{
  int * fileSizes;
  int * fileOwners;
  char ** files;
};

typedef struct VirtualHDD VirtualHDD;

//Variables used as part of storing and processing data.

extern int memSize;  //The current memory size
extern int numPagesPerSegment;  //The number of pages per segment
extern int frameSize;  //The size of each frame in KB
extern int numFrames;  //The number of frames in physical RAM
extern VirtualHDD hdd; //The virtual hard disk.  This holds all files.
extern int numFiles;   //The number of files in the virtual hard disk.
extern PhysicalRam ram; //The physical RAM.  This is made up of frames.
extern Segment * segmentTable; //The list of segments (a linked list).

//Mutual exclusion variables

/********** Add variables for mutual exclusion here ************/
extern sem_t memoryLock;  //This variable is only an example.
                          //You could use a mutex and condition variable.


//Function prototypes

void init();
void requestMemory();
void releaseMemory();
void cleanUp();

#endif
